%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ENGO 638 - 2012                                                         %
% Lab 1 - The acquisition stage                                           %
% Instructor: Daniele Borio                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Signal Characteristics                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fs = 5e6;       % Sampling frequency
fi = 0.42e6;    % Intermediate frequency
IsComplex = true;  % Tell if the samples are in I/Q format
Is16Bits  = true;  % Tell if the samples are stored on 16 bits/byte

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Acquisition Properties                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nd = 81;        % Number of Doppler bins
DopStep = 125;  % Doppler bin size in Hz
secondOfData = 0.1; % s - Seconds of data to read

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Open the input file                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen ('../../GpsL116bitCpx_1575M5M_01Aug10.bin','r');

if( IsComplex ),
    if( Is16Bits ),
        [data, cnt_data] = fread(fid, 2 * secondOfData * fs, 'int16');
    else
        [data, cnt_data] = fread(fid, 2 * secondOfData * fs, 'int8');
    end
    data = data(1:2:end) + 1i * data(2:2:end);
else
    if( Is16Bits ),
        [data, cnt_data] = fread(fid, secondOfData * fs, 'int16');
    else
        [data, cnt_data] = fread(fid, secondOfData * fs, 'int8');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%          Generate the local code and resample it                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sat = 28;       % PRN of the satellite to acquire
loc = GpsCaCode( sat ); % The code is in binary format - need to be converted
                        % into a bipolar format
locBipolar = 1 - 2 * loc;   % Code in bipolar format

fc = 1.023e6;   % Code rate of the code
Tc = 0.001;     % Coherent integration time in ms
Nc = Tc * fs;
locC = ResampleCode( locBipolar, Nc, fs, 0, fc );   % Resample the code at
                                                    % data sampling
                                                    % frequency

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Main acquisition loop - evaluation of the search space         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K = 8;  % Number of non-coherent integrations
sspace = 0;                 % Search space were the results will be stored
codeDl = (0:(Nc - 1)) / fs; % Code delays

for ii = 1:K,
    y =  data( (ii - 1) * Nc + (1:Nc) ).';   % use just 1 period of code at the time
    
    % Compute the search space for a single coherent integration epoch
    Tsspace = DftParallelCodePhaseAcquisition( y, locC, Nc, Nd, DopStep, fs, fi );
    
    sspace = sspace + Tsspace;  % Non-coherently accumulate the results 
end 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             Add decision logic here                            % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Pfa_sys = 0.1;                                              % System Probability
Pfa_cell = 1 - ( 1 - Pfa_sys ).^( 1 / numel( sspace ) );    % Cell Probability    

% Noise floor estimation and threshold evaluation
Th = InverseChiSquarePfa( K, Pfa_cell );                    % Normalized Threshold

sigma2 = NoiseVarianceEstimator( y, fs, fc, fi );          % Noise floor estimator

Th = sigma2 * Th;                                           % Unnormalized variance

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                               Plot results                                     % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure
surf( codeDl, ((1:Nd) - ceil(Nd/2))*DopStep, sspace, 'EdgeColor', 'none');
axis tight
xlabel('code delay (samples)')
ylabel('frequencies (Hz)')

% Add the threshold here
hold on
surf( [ codeDl(1) codeDl(end) ], [ (1 - ceil(Nd/2))*DopStep (Nd - ceil(Nd/2))*DopStep ], ...
        Th * ones(2,2), 'FaceAlpha', 0.2, 'FaceColor', 'r' );
hold off

% Find the Doppler frequency
figure
if bitand( Nd, 1 ) == 1,    % It is an odd number 
    Freq = (-((Nd - 1) / 2):((Nd - 1) / 2) ) * DopStep;
else
    Freq = (-(Nd/2):( (Nd-2) / 2 ) ) * DopStep;
end

[maxVal DopInd] = max(max(sspace.'));
DopFreq = Freq(DopInd)
plot( codeDl, sspace(DopInd, :) / maxVal )
axis tight
grid on
set( gca, 'FontSize', 16)
xlabel('Code delay (samples)')
ylabel('Normalized Correlation')

% Add the threshold here
hold on
plot( [ codeDl(1) codeDl(end) ], Th * ones(1,2) / maxVal, 'r', 'LineWidth', 2 );
hold off


% Find the code delay (in chips)
figure
[maxVal codInd] = max(max(sspace));
codeDelay = 1023 - (codInd - 1) / fs * fc
% codeDelay = codInd / fs * fc

plot(Freq, sspace(:, codInd) / maxVal)
axis tight
grid on
set( gca, 'FontSize', 16)
xlabel('Doppler frequency (Hz)')
ylabel('Normalized Correlation')

hold on
plot( [ Freq(1) Freq(end) ], Th * ones(1,2) / maxVal, 'r', 'LineWidth', 2 );
hold off

% Final decision
if maxVal > Th
    disp('PRN present');
end